package VocTrainer::Params;


###########################################################################
#
#  a part of "Weird Web v0.2" a web publishing system (framework)
#  Copyright (C) 2002 koppi@weirdoz.net
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#  
###########################################################################


use strict;
use VocTrainer::Utils;

require Exporter;
use vars qw(@ISA @EXPORT);
use constant DEBUG => 0;
@ISA = qw(Exporter);
@EXPORT = qw(&GetParams);


=head1 NAME

VocTrainer::Params

=head1 SYNOPSIS

use VocTrainer::Params;

my %in = %{ GetParams( QUERY =E<gt> $q};


=head1 DESCRIPTION

Reads Params from CGI query and returns \%in 

Requires VocTrainer::Utils
 
=head1 FUNCTIONS


=over 4


=item B<GetParams>

Reads Params from CGI query and returns values in \%in 

Allows funny data structures, which can be used like this: 
<input type="text" name="array[0][1]" value="string">
=> $in{array}[0][1] = 'string'

If U wanna use these funny structures your param-names have 
to match

C</^(\w+)((?:(?:\[\d+\])|(?:\{\w+\}))+)$/>

Returns a hash with all your cgi parameters. Additionally, the following
keys are defined (these keys are to be consideres as reserved words, you
can't use them for your own values):
 C<ACTION_NAME> name of the action that should 
               be executed (scalar)
 C<ACTION_ARGV> arrayref to the arguments for 
               the action
 C<ACTION_ARGC> number of the arguments 
               in ACTION_ARGV


B<Required params:>
 QUERY  => $CGI::Object

B<Optional params:>
 FORMAT => html|unicode (allows conversion to 
                         html or unicode)

=back

=cut

#######################################################################
# get_params( CGI->query )                                            #
# read all params of CGI->$query and set variables named $param       #
# with $params value in the scope of the calling script               #
#######################################################################

sub GetParams 
{
	my %args = (
		QUERY => '',
		FORMAT => '',
		@_
	);

	my %in = ();				# hash containing name => value pairs
	unless ($args{QUERY}) {
		return (wantarray) ? %in : \%in;
	}

	my $query = $args{QUERY};
	my @params = $query->param;	# array containing names of all params

	foreach my $name (@params) {
		my $value = $query->param($name);
		#dprint ("$name ... $value") if DEBUG;
	#	print " *** ";
	#	dprint $value;

		$name =~ s/\.[xy]$//i;

		### param is an action
		if ($name =~ /^action_/) {
			$name =~ s/\.[xy]$//i;
			@{$in{action}} = split(/_/, $name);
			shift @{$in{action}};	
		### params is part of an array

		} elsif ($name =~ /^(\w+)((?:(?:\[\d+\])|(?:\{\w+\}))+)$/) {	

			##### $1 ... "myarr"
			##### $2 ... "[1]{3}[4]"
			dprint ("$1 ......... $2") if DEBUG;
#			my $code = "\$in{$1}$2 = \'" . quotemeta($value) . "\'";
			# SECURE ????
			# koppi scheisst drauf
#			eval qq|\$in{$1}$2 = <<EOF
#$value
#EOF|;
			my $arr = $1;
			my $tmp = $2;
			#### hash keys that start with a \d have to be quoted,
			#### because perl interpretes the string 1_123 as
			#### the numerical value 1123
			$tmp =~ s/\{(\d[^}]*)\}/\{\"$1\"\}/g;
			my $foo = "\$in{$arr}" . "$tmp" ;
			if (!ref $value) {
				if ($args{FORMAT}) {
					$value = VocTrainer::Utils::ascii2html( $value ) if ($args{FORMAT} eq "html");
					$value = VocTrainer::Utils::ascii2htmlform( $value ) if ($args{FORMAT} eq "htmlform");
					$value = VocTrainer::Utils::ascii2unicode( $value ) if ($args{FORMAT} eq "unicode");
				} else {
					$value = VocTrainer::Utils::chompdata($value);
				}

				$foo .=  " = \"" . quotemeta($value) . "\""; ## \"$value\"";
				dprint ("--- $foo") if DEBUG;
				#### NEW NEW NEW NEW NEW NEW NE WNEW
				#### allend variable within in %in drin
				# allend($arr, $tmp, \%in, $value);
				#### NEW NEW NEW NEW NEW NEW NE WNEW
			} else {
				#### now image upload files are handled 'somehow'
				use Data::Dumper;
 				my $bertl = Data::Dumper::Dumper($value);
				$bertl =~ s/^\$VAR1 = //;
				$foo .= " = $bertl";
				# print $foo;
			}
			### hit me baby one more time ###
			no strict "refs";
			### *ouch* ###
			eval  "$foo" ; 
			### satan ###
		### params is				
		### checkbox - kommt weg sagt sc
#		} elsif ($name =~ /^(.+)\[(\d+)\]$/) {
#			$in{$1}[$2] = $value;
		### param is just a scalar
		} else {							
			if ($args{FORMAT}) {
				$value = VocTrainer::Utils::ascii2html( $value ) if ($args{FORMAT} eq "html");
				$value = VocTrainer::Utils::ascii2htmlform( $value ) if ($args{FORMAT} eq "htmlform");
				$value = VocTrainer::Utils::ascii2unicode( $value ) if ($args{FORMAT} eq "unicode");
			} else {
				$value = VocTrainer::Utils::chompdata($value);
			}
			$in{$name} = $value;
		}
	}

=comment
	if ($args{FORMAT}) {
		%in = %{ VocTrainer::Utils::ascii2html( \%in ) } if ($args{FORMAT} eq "html");
		%in = %{ VocTrainer::Utils::ascii2unicode( \%in ) } if ($args{FORMAT} eq "unicode");
	} else {
		%in = %{ VocTrainer::Utils::chompdata (\%in) };
	}

=cut

	#### eine kleine idee zur erleichterung der actions 
	if (defined $in{action} and (ref($in{action}) eq 'ARRAY')) {
		$in{ACTION_NAME} = $in{action}->[0] || "";
		@{$in{ACTION_ARGV}} = @{$in{action}};
		shift(@{$in{ACTION_ARGV}});
		$in{ACTION_ARGC} = ($#{$in{ACTION_ARGV}} + 1) || 0;
		if (DEBUG) {
			dprint ("[ACTION_NAME] $in{ACTION_NAME}<br>") if DEBUG;
			dprint ("[ACTION_ARGV] (" . join(", ", @{$in{ACTION_ARGV}}) . ")") if DEBUG;
			dprint ("[ACTION_ARGC] $in{ACTION_ARGC} Werte") if DEBUG;
		}
	}



	return  (wantarray) ? %in : \%in;
}

1;



=head1 BUGS

None known yet.

=head1 SEE ALSO

L<CGI> and other Weird Web documentation

=head1 AUTHORS

slayven@xover.mud.at, sc@gimp.org, koppi@weirdoz.org
 
=cut

