package VocTrainer::Utils;


###########################################################################
#
#  a part of "Weird Web v0.2" a web publishing system (framework)
#  Copyright (C) 2002 koppi@weirdoz.net
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#  
###########################################################################


require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(dprint ascii2html html2ascii ascii2unicode readfile convdate);

=head1 NAME 

VocTrainer::Utils

=head1 SYNOPSIS 

use VocTrainer::Utils;

=head1 DESCRIPTION 

Utilities Library, provides a set of small functions
that are used on a regular basis.

=cut


use strict;
use CGI;
# use CGI::Carp qw/fatalsToBrowser/;
use Unicode::String qw(latin1 utf8);
use URI::Escape;
use HTML::Entities;
use Data::Dumper;

use Term::ANSIColor;

use constant DEBUG => 0;
use constant DEBUGDEST => "STDERR";

$VocTrainer::Utils::debug = 0;

use vars qw(%repl);

%repl = (
	""	=>	"ae",
	""	=>	"Ae",
	""	=>	"oe",
	""	=>	"Oe",
	""	=>	"ue",
	""	=>	"Ue",
	""	=>	"sz",	
);

1;

=head1 FUNCTIONS

The following utility functions are provided:

=cut



sub handle_warning
{
    my $warning = shift;
    print qq(<font color="#ff0000">$warning</font><br>);
}

sub handle_error
{
    my $error = $_[0];
    print STDERR qq(***ERROR: $error\n);    
#	exit(0);
}

sub handle_dberror
{
    my $error = $_[0];
    my $dbh = $_[1];
    print STDERR qq(***ERROR : $error\n);
	die "DB ERROR!!";
	return "SHIT";
#    exit(0);
}


sub convdate
{
	my %args = (
		DATE => '',
		FORMAT => 'DE',
		DS => 'string',
		DELIMITER => '',
		@_
	);

	return "" unless $args{DATE};

	my @date;

	if (ref $args{DATE} eq 'ARRAY') {
		@date = @{$args{DATE}};
	} elsif (ref $args{DATE} eq 'HASH') { 
		@date = ($args{DATE}{day}, $args{DATE}{month}, $args{DATE}{year});
		if (exists ($args{DATE}{time})) {
			push (@date, $args{DATE}{time});
		}
	} else {
		$date[0] = $args{DATE};
	}


	if ($#date == 0) {
		if ($date[0] =~ /[\-\s\.]/) {
			@date = split (/[\-\s\.]/, $date[0]);
		} elsif ($date[0] =~ /^\d{14}$/) {
			my $d = $date[0];
			@date = (
				substr($d, 6,2), 
				substr($d, 4,2), 
				substr($d, 0,4), 
				substr($d, 8,2) . ":" . substr($d,10,2) . ":" . substr($d,12,2)
			);
		} elsif ($date[0] =~ /^\d{8}$/) {
			my $d = $date[0];
			@date = (
				substr($d, 6,2), 
				substr($d, 4,2), 
				substr($d, 0,4), 
			);
		}
	}

	my ($d, $m, $y, $t) = ("","","","");
	
	if ( $date[3] && $date[0] =~ /\:/ ) {
		$t = shift(@date);
	}

	if (length($date[0]) == 4) {
		($y, $m, $d) = @date[0,1,2];
	} elsif (length($date[2]) == 4) {
		($y, $m, $d) = @date[2,1,0];
	}
	$t ||= $date[3];

	my $delimiter = $args{DELIMITER} || '';
	my @rdate;

	if (uc($args{FORMAT}) eq 'EN') {
		$delimiter ||= '-';
		@rdate = ($y, $m, $d);
	} else {
		$delimiter ||= '.';
		@rdate = ($d, $m, $y);
	}
	
	if (uc($args{DS}) ne 'ARRAY') {
		my $r = join ( $delimiter, @rdate );
		$r .= ($t) ? " $t" : '';
		return $r;
	} else {
		if ($t) {
			push (@rdate, $t);
		}
		return (wantarray) ? @rdate : \@rdate;
	}
}


sub to_unicode
{
    my $str = html2ascii($_[0]);
    return "" unless $str;

    $str =~ s/\&/\&\#038\;/g;
    $str =~ s/\"/\&\#034\;/g;
    ### fraglich
    # print STDERR $str, $/;
    my $u = latin1($str)->utf8;
    $u =~ s/</#lt#/g;
    $u =~ s/>/#gt#/g;
    # $u =~ s/</\&\#19252\;/g;
    $u =~ s/\r?\n/\<![CDATA[<br>]]>\n/g;
    return $u;
}

sub unicode2html {
	my $unicode = shift;
	my $context = shift || "";
#	print STDERR "----------------------- UNICODE --------------------\n";
#	print STDERR $unicode;
#	print STDERR "----------------------- /UNICODE --------------------\n";
	my $ascii = unicode2ascii($unicode, $context);
#	print STDERR "----------------------- ASCII --------------------\n";
#	print STDERR $ascii;
##	print STDERR "----------------------- /ASCII --------------------\n";
	
#	my $html = html2html($ascii);
#	return $html;
	return $ascii;
}

sub html2html {
	my $html = shift;

	return $html;
}

sub make_links
{
    my $link = $_[0];
#    $link =~ s/([a-zA-Z_0-9.\-~]+.\@.[a-zA-Z_0-9.\-]+\..[a-zA-Z_0-9.\-]+)/<a href=\"mailto:$1\">$1<\/a>/g;
#    $link =~ s/([a-zA-Z_0-9.\-~]+\@[a-zA-Z_0-9.\-]+\.\w{2,4})/<a href=\"mailto:$1\">$1<\/a>/g;
#	if (
#	$link =~ s/(?<!\[link:mailto:)([a-zA-Z_0-9.\-~]+.\@.[a-zA-Z_0-9.\-]+\..[a-zA-Z_0-9.\-]+)/<a href=\"mailto:$1\">$1<\/a>/g;
	$link =~ s/\[link:([^\]]+)\]([^\[]+)\[\/link\]/&makelink($1,$2);/emg;
#    $link =~ s/(http:\/\/[a-zA-Z_0-9.\/\-~\?\=\&\#\;]+)/<a href=\"$1\" target=\"extern\">$1<\/a>/g;
    return $link;
}

sub makelink {
	my $uri = shift;
	my $txt = shift;
	my $target;
	if ($uri =~ /^http:/) {
		$target = "extern"
	}
	return qq|<a href="$uri" | . ( ($target) ? "target=\"$target\"" : "" ) . qq|>$txt</a>|;
}

sub findwords {
	my %args = (TXT => '',
				@_);
	my %links = (
		'Zwangssterilisierungen'	=> "view?object=Thema;id=5",
		'Aktion T4'	=> "view?object=Thema;id=7",
		'Euthanasie' => "view?object=",
	);

	foreach (keys %links) {
		$args{TXT} =~ s/$_/<a href="$links{$_}">$_<\/a>/;
	}
	return $args{TXT};	
}

=head2 STRING FUNCTIONS

=over 4

=item B<choptext>

chop chop chop!

=cut

sub choptext
{
    my $text = $_[0];
    my @lines;
    my $lastline = 0;
    my $returntext = "";
    @lines = split(/\n/, $text);
    for (my $i=$#lines; $i>0; $i--)
    {
		$lines[$i] =~ s/^\s+$//g;
		if ($lines[$i] ne "" && $lines[$i] !~ /^[\r\n]/)
		{
	    	$lastline = $i;
	    	last;
		}
    }
    foreach my $i (0..$lastline)
    {
		chomp $lines[$i];
		$lines[$i] =~ s/\s+$//;
		$returntext .= $lines[$i] . "\n";
    }
	chomp $returntext;
    return $returntext;
}

=item B<cut_text( $text, $length )>

Cuts the string in $text to a length of $length characters or less,
not truncating any words, appends '...' and returns it.

=cut

sub cut_text 
{
    my ($text, $text_length) = @_;

    $text =~ s/<br>\n/\n/g;
    if (length($text) > $text_length) {
		my @tmp=split(/ /,substr($text,0,$text_length));
		if (($#tmp) > 0) {
		    pop(@tmp);
		}
		$text = join(" ",@tmp) . "...";
    }
    return $text;
}

=back

=head2 ENCODING FUNCTIONS

=over 4

=item B<ascii2html( $scalar || $hashref || $arrayref)>

Encodes characters into html format. If argument is a string,
it just converts the string. If argument is a reference to 
hash or array, the function subsequently calls itself on
each of the data structure's contents. The return value is
the encoded string or the reference to the encoded data
structure. 


=item B<html2ascii ( $scalar || $hashref || $arrayref)>

Decodes htmlentites to ascii characters.

=cut

sub ascii2htmlform {
	my $ascii = shift;
	return &ascii2html($ascii, "nobr");
}

sub ascii2html {
	my $ascii = shift;
	my $nobr = shift || "";

	return "" unless (defined $ascii );

	if (! ref $ascii) {
		print STDERR "SCALAR\n" if DEBUG;
		my $html = encode_entities($ascii);
		$html =~ s/\n/<br>\n/g unless $nobr;
		return $html;
	}
	
	elsif (ref $ascii eq "ARRAY") {
		print STDERR "ARRAY\n" if DEBUG;
		foreach my $i (0..$#{$ascii}) {
			# print STDERR "REF: " . ( ref ${$ascii}[$i]) . "xxx\n";
			${$ascii}[$i] = ascii2html(${$ascii}[$i], $nobr);      
		}
		return $ascii;
	}
	
	elsif (ref $ascii eq "HASH") {
		print STDERR "HASH\n" if DEBUG;
		foreach my $k (keys %{$ascii}) {
			${$ascii}{$k} = ascii2html(${$ascii}{$k}, $nobr)
		}
		return $ascii;
	}
}

sub html2ascii {

	my $html = shift;
	return "" unless (defined $html );

	if (! ref $html) {
		my $ascii = decode_entities($html);
		$ascii =~ s/<br>.?\n/\n/g;
		return $ascii ;
	}
	
	elsif (ref $html eq "ARRAY") {
		print STDERR "ARRAY\n" if DEBUG;
		foreach my $i (0..$#{$html}) {
			# print STDERR "REF: " . ( ref ${$ascii}[$i]) . "xxx\n";
			${$html}[$i] = html2ascii(${$html}[$i]);      
		}
		return $html;
	}
	
	elsif (ref $html eq "HASH") {
		print STDERR "HASH\n" if DEBUG;
		foreach my $k (keys %{$html}) {
			${$html}{$k} = html2ascii(${$html}{$k})
		}
		return $html;
	}

}

sub unicode2ascii {
	my $unicode = shift;
	my $context = shift || "";
	return "" unless (defined $unicode);
	if (! ref $unicode) {
		print STDERR "SCALAR $unicode\n" if DEBUG;
	    my $u = utf8($unicode);
    	my $str;
		if ($u) {
			$str = $u->latin1;
		} else {
			$str = $unicode;       
		}
		$str ||= "";
    	$str =~  s/\&amp;/\&/g;

    	$str =~ s/\&\#19252\;/&lt;/g;
    	$str =~ s/#lt#/&lt;/g;
	    $str =~ s/\&lt;/</g; # unless $context eq "view";

    	$str =~ s/\&\#19253\;/&gt;/g;
    	$str =~ s/#gt#/&gt;/g;
	    $str =~ s/\&gt;/>/g; #  unless $context eq "view";

		$str =~ s/<\!\[CDATA\[\<br\>\]\]\>//g;

		$str =~ s/(\&\#\d+;)/&mychr($1)/eg;

		return $str;
	}
	
	elsif (ref $unicode eq "ARRAY") {
		print STDERR "ARRAY\n" if DEBUG;
		foreach my $i (0..$#{$unicode}) {
			# print STDERR "REF: " . ( ref ${$ascii}[$i]) . "xxx\n";
			${$unicode}[$i] = unicode2ascii(${$unicode}[$i]);      
		}
		return $unicode;
	}
	
	elsif (ref $unicode eq "HASH") {
		print STDERR "HASH\n" if DEBUG;
		foreach my $k (keys %{$unicode}) {
			${$unicode}{$k} = unicode2ascii(${$unicode}{$k})
		}
		return $unicode;
	}
}

sub mychr {
	my $num = shift;
	my $chrnum = $num;
	$chrnum =~ s/\D//g;
#	print STDERR "mychr:  $num ... $chrnum ... " . chr($chrnum) . "\n";
	# anfuehrungszeichen oben/unten
	if ($chrnum == 8222 || $chrnum == 8220) {
		return '"';
	# einfache anfuehrungszeichen oben/unten
	} elsif ( $chrnum == 8216 || $chrnum == 8218) {
		return "'";
	} else {
		return chr($chrnum) || $num;
	}
}
sub ascii2unicode {
	my $ascii = shift;

	return "" unless (defined $ascii );

	if (! ref $ascii) {
		print STDERR "SCALAR\n" if DEBUG;
		my $uni = to_unicode($ascii);
		return $uni;
	}
	
	elsif (ref $ascii eq "ARRAY") {
		print STDERR "ARRAY\n" if DEBUG;
		foreach my $i (0..$#{$ascii}) {
			# print STDERR "REF: " . ( ref ${$ascii}[$i]) . "xxx\n";
			${$ascii}[$i] = ascii2unicode(${$ascii}[$i]);      
		}
		return $ascii;
	}
	
	elsif (ref $ascii eq "HASH") {
		print STDERR "HASH\n" if DEBUG;
		foreach my $k (keys %{$ascii}) {
			${$ascii}{$k} = ascii2unicode(${$ascii}{$k})
		}
		return $ascii;
	}
}


sub chompdata {
	my $data = shift;
	return unless (defined $data );

	if (! ref $data) {
		chomp $data;		
		return $data;
	}
	
	elsif (ref $data eq "ARRAY") {
		foreach my $i (0..$#{$data}) {
			${$data}[$i] = &chompdata(${$data}[$i]);      
		}
		return $data;
	}
	
	elsif (ref $data eq "HASH") {
		foreach my $k (keys %{$data}) {
			${$data}{$k} = &chompdata(${$data}{$k})
		}
		return $data;
	} else {
		### Filehandles und so schmaehs ...
		return $data;
	}
}


=item B<encodeurl( $scalar )>

Encodes string in uri format and returns it

=back 

=cut

sub encodeurl
{
    my $str = $_[0];
    return unless $str;
    my $enc = uri_escape($str);
    return $enc;
}

sub txt2url {
	my $txt = shift;
	$txt = unicode2ascii($txt);
	foreach (keys %repl) {
		$txt =~ s/$_/$repl{$_}/g;
	}
	$txt =~ s#\W+#_#g;
	return encodeurl($txt);
}

sub url2txt {
	my $url = shift;
	$url = unicode2ascii(uri_unescape($url));
	foreach (keys %repl) {
		$url =~ s/$repl{$_}/$_/g;
	}
	$url =~ s/\_/\%/g;
	return $url;
}
=head2 DATE FUNCTIONS

=over 4

=item B<convdate>

 converts dates and returns a string

 Required Params: 
   DATE       arrayref, hash or string

 Optional Params: 
   FORMAT     so far only 'en' is supported (YYYY-MM-DD)
              anything else converts date into DD.MM.YYYY

   DELIMITER  default value is '.'
              useless if FORMAT is given and set to 'en'
              
   DS         'array' 
              returns an array in list context.
              returns arrayref in scalar context.

=back

=head2 DEBUG FUNCTIONS

=over 4

=item B<debugxml( $xmltext )>

Prints the contents of $xmltext to F<E<lt>STDOUTE<gt>>. Additionally, 
htmlencodes the string and prepends line numbers in front
of each line for debugging, and wraps the whole thing inside
a E<lt>preE<gt> tag.

=item B<dprint ( $debug )>

Prints the $debug string or dumps datastructure if $debug id a reference.

=cut

sub debugxml
{
    my $xml = $_[0];
    $xml =~ s/</&lt;/g;
    $xml =~ s/>/&gt;/g;

    print "Content-type: text/html\n\n\n<pre>";
    my @lines = split(/\n/, $xml);
    my $nr = 1;
    foreach my $i(0..$#lines) {
        if ($lines[$i] =~ /^-----------------------------/) {
            $nr = 0;
        }
        print "" . ($nr++) . "\t" . $lines[$i] . "\n";
    }
    print "</pre>";
}


sub dprint
{
	my $d = shift;

 	my ($package, $filename, $line) = caller;
	my $info;
	my $dest = "";
	($info = $filename) =~ s/^.*\/([^\/].*)$/$1/;
	
	my $colored = 1;
	my $color = "red";
#	if (declared ("main::COLOR")) {
#		$colored = main::COLOR();
#	}

	if ($colored) {
		my $foo = 0;
		$foo += ord($_) foreach (split //, $info);
		$foo = int ($foo / 23);
		my @colors = ("red", "blue", "green", "black", "cyan", "magenta");
		$color = $colors[($foo % $#colors)];
	}
#	if (main::DEBUG() > 1) {
		$info .= " || $line ";
#	}
#	if (declared ("main::DEST")) {
#		$dest = main::DEST();
#	}
	$dest = DEBUGDEST;
	$color = "blue" if ($d && $d =~ /^\*\*\*/);
	if (exists $ENV{TERM} || $dest) {
		$dest ||= "STDOUT";
		my $oldfh = select($dest); $|++;
		print color "$color";
		print "[ $info ] ";
		if (ref $d) {
			print Dumper ($d);
		} else {
			print $d;
		}
		print "\n";
		print color 'reset';
		select ($oldfh);
	} else {
		print qq|<font style="color:$color;">[ $info ] |;
		if (defined $d && ref $d) {
			print "<hr><pre>" . Dumper ($d) . "</pre><hr>";
		} else {
			print $d;
		}
		print "</font><br>";
	}
}

=comment
sub declared ($) {
	use constant 1.01;              # don't omit this!
	my $name = shift;
	$name =~ s/^::/main::/;
	my $pkg = caller;
	my $full_name = $name =~ /::/ ? $name : "${pkg}::$name";
	$constant::declared{$full_name};
}
=cut
sub readfile {   
    my $file = $_[0];
    my $data = "";
    open (F, "$file") || die "Can't open $file";
    while (<F>)
    {
        $data .= $_;
    }
    return $data;
}


=back 

=head1 AUTHORS

da koppi, da slay und i 1997-2001


