#!/usr/bin/perl -w 

use strict;
use CGI;
use locale;
use POSIX qw(locale_h setlocale LC_ALL LC_CTYPE);


use VocTrainer::Database;
use VocTrainer::Layout;
use VocTrainer::Users;
use VocTrainer::Params;
use VocTrainer::Utils;


# TODO get from db
my @languages = ('de', 'es');
my %langstrings = (
	"de" => "Deutsch",
	"es" => "Espaol"
);

my %labels = (
	word => "Wort / Phrase",
	article => "Artikel",
	typeid => "Type",
	remark => "Anmerkung",
	pronunciation => "Aussprache",
	antonyms => "Antonyme",
	synonyms => "Synonyme",
	example => "Beispiel",
	paraphrase => "Umschreibung",
	entryusage => "Gebrauch",
	conj => "Konjugation"
);

my $q = new CGI;
my %in = GetParams(QUERY => $q);

# dprint(\%in);

$in{lang} ||= $languages[0];
$in{sortlang} ||= $in{lang};
$in{desc} ||= 0;
$in{q} ||= "";				# query
$in{view} =~ s/\D//g if $in{view};
$in{edit} =~ s/\D//g if ($in{edit} && $in{edit} ne "new");
$in{del}  =~ s/\D//g if $in{del};

my $lctype = $in{sortlang} . "_" . uc($in{sortlang});
my $loc = setlocale(LC_ALL, $lctype);

my @levels;

my $dbh = DB_open();


my $user = VocTrainer::Users::CheckUser(DBH => $dbh);

print $q->header();

# dprint($user);

my $w = ($in{small}) ? 600 : 600;
&VocTrainer::Layout::printhtmlheader(TITLE => "W&ouml;rterbuch" , USER => $user, WIDTH => $w);


&searchform unless $in{small};

if ($in{view} || $in{edit}) {
	@levels = &get_levels();
}

if ($in{view}) {
	&viewentry($in{view});
} elsif ($in{edit}) {
	if ($user->{isadmin}) {
		if ($in{save}) {
			my $error = &checkinput;
			if ($error) {
				&editentry($in{edit}, $error);
			} else {
				&saveentry($in{edit});
				&editentry($in{edit}, "", "Eintrag wurde gespeichert");
			}
		} else {
			&editentry($in{edit});
		}
	} else {

	}
} elsif ($in{del} && $user->{isadmin}) {
	# delete entry
	&delentry($in{del});
} else {

	if ($in{q}) {
		my $entries = &get_entries();
		if ($entries) {
			&print_list($entries, $user);
		} else {
			print qq|<table border="0" cellspacing="0" cellpadding="3" width="$w">
<tr>
	<td>Leider keine Entr&auml;ge gefunden</td>
</tr>
</table>
|;

		}
	}
}

DB_close(DBH => $dbh);

sub get_one_entry {
	my $id = shift;
	my $stmt = "select id, langcode, word, article, typeid, remark, pronunciation, antonyms, synonyms, example, paraphrase, entryusage from Entries where id = '$id'";
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);

	my %entries;

	foreach (@data) {
		$entries{$_->[1]} = {
			langcode => $_->[1],
			word => $_->[2],
			article => $_->[3],
			typeid => $_->[4],
			remark => $_->[5],
			pronunciation => $_->[6],
			antonyms => $_->[7],
			synonyms => $_->[8],
			example => $_->[9],
			paraphrase => $_->[10],
			entryusage => $_->[11]
		}; 
	} 

	my $cstmt = "select entryid, langcode, conjugationcode, conjugation from Conjugations where entryid = $id";
	my @cdata = DB_getdata(STMT => $cstmt, DBH => $dbh);

	foreach (@cdata) {
		$entries{$_->[1]}->{conj}->{$_->[2]} = $_->[3];
	}


	my $lstmt = "select levelid from Entries_in_Levels where entryid = $id";
	my @ldata = DB_getdata(STMT => $lstmt, DBH => $dbh, ONECOLUMN => 1);
	$entries{$languages[0]}->{level} = $ldata[0] || 21;

	return VocTrainer::Utils::unicode2html(\%entries); 
}

sub viewentry {
	my $id = shift;
	my $info = shift;

	my %entries = %{ get_one_entry($id) };
	print qq|<br><table border="0" cellspacing="0" cellpadding="3" width="$w">|;
	if ($info) {
		print qq|<tr>
<td colspan="3">$info</td>
</tr>\n|;
	}
	print qq|<tr class="tableheader" width="100">|;
	print qq|<td class="tableheader">&nbsp;</td>\n|;
	foreach (@languages) {
		print qq|<td class="tableheader" width="250">$langstrings{$_}</td>\n|;
	}
	print qq|</tr>|;
	foreach my $e (qw(word remark pronunciation antonyms synonyms example paraphrase entryusage)) {
		my $hasvalue = 0;
		foreach (@languages) { 
			if ($entries{$_}->{$e}) {
				$hasvalue = 1;
				last;
			}
		}
		if ($hasvalue) {
			print qq|<tr>|;
			print qq|<td class="viewlabel" valign="top">$labels{$e}:</td>|;
			my $class = "normale";
			$class = "theword" if $e eq "word";
			foreach (@languages) {
				print qq|<td valign="top" class="$class">|;
				if ($e eq "word" && $entries{$_}->{article}) {
					print $entries{$_}->{article} . qq| |;
				}
				print $entries{$_}->{$e} . qq|</td>\n|;
			}
			print qq|</tr>|;
		}
	}

#	print qq|<tr><td colspan="3" class="viewlabel">&nbsp;</td></tr>\n|;
	my $hasconj = 0;
	foreach (@languages) { 
		if ($entries{$_}->{conj}) {
			$hasconj = 1;
			last;
		}
	}
	if ($hasconj) {
		my $pronouns = &get_pronouns(DBH => $dbh);
		my $conjcodes = &get_conjcodes(DBH => $dbh);
		print qq|<tr>|;
		print qq|<td class="viewlabel" valign="top">$labels{conj}:</td>|;
		foreach my $l (@languages) {
			print qq|<td valign="top">|;
			if ($entries{$l}->{conj}) {
				print qq|<table width="250" border="0" cellpadding="0" cellspacing="0">|;
				my %con = %{ $entries{$l}->{conj} };
				my @codes = qw(s1 s2 s3f s3m s3n p1 p2 p3f p3m p3n);
				foreach my $c(@codes) {
					if ($con{$c}) {
						print qq|<tr>
	<td>| . $conjcodes->{$c} . qq|</td>
	<td>| . $pronouns->{$c}->{$l} . qq|</td>
	<td>$con{$c}</td>
</tr>\n|;
					}
				}
				print qq|</table>|;
			} else {
				print "&nbsp;";
			}
			print qq|</td>|;
		}
		print qq|</tr>|;
	}

	if ($in{small}) {
		print qq|<tr>
	<td colspan="3" align="center"><br>
	<a href="javascript:window.close()">schliessen</a>
	</d>
</tr>
|;
	} else {
		print qq|<tr>
	<td><br><a href="javascript:history.back()">&laquo; zur&uuml;ck</a></td>
	<td></td>
	<td  align="right"><br>|;
		if ($user->{isadmin}) {
			print qq|<a href="dict.pl?del=$id">del</a> &nbsp;|;
			print qq|<a href="dict.pl?edit=$id">edit</a>|;
		} 
		print qq|</td>
</tr>
|;
	} 

	print qq|</table>|;
}


sub editentry {
	my $id = shift;
	my $error = shift;
	my $info = shift;

	my %entries = ();
	if ($in{entries}) {
		%entries = %{$in{entries}};
	} else {
		%entries = %{ get_one_entry($id) } unless $id eq "new";
	}
	print qq|<form method="POST">\n|;
	print qq|<input type="hidden" name="edit" value="$id">\n|;
	print qq|<input type="hidden" name="save" value="1">\n|;
	print qq|<table border="0" cellspacing="0" cellpadding="3" width="$w">\n|;
	if ($error) {
		print qq|<tr>
	<td colspan="3" class="error">$error<br></td>
</tr>\n|;

	}
	if ($info) {
		print qq|<tr>
	<td colspan="3">$info<br></td>
</tr>\n|;

	}
	print qq|<tr class="tableheader" width="100">\n|;
	print qq|<td class="tableheader">&nbsp;</td>\n|;
	foreach (@languages) {
		print qq|<td class="tableheader" width="250">$langstrings{$_}</td>\n|;
	}
	print qq|</tr>\n|;
	foreach my $e (qw(word article remark pronunciation antonyms synonyms example paraphrase entryusage)) {
		print qq|<tr>\n|;
		print qq|<td class="viewlabel" valign="top">$labels{$e}:</td>\n|;
		my $class = "normale";
		$class = "theword" if $e eq "word";
		foreach (@languages) {
			print qq|<td valign="top" class="$class">|;
			my $val = $entries{$_}->{$e} || "";
			print qq|<input type="text" name="entries{$_}{$e}" value="$val" size="30">|;
			print qq|</td>\n|;
		}
		print qq|</tr>\n|;
	}

#	print qq|<tr><td colspan="3" class="viewlabel">&nbsp;</td></tr>\n|;
#	my $pronouns = &get_pronouns(DBH => $dbh);
	my $conjcodes = &get_conjcodes(DBH => $dbh);
	print qq|<tr>\n|;
	print qq|<td class="viewlabel" valign="top">$labels{conj}:</td>\n|;
	my @codes = qw(s1 s2 s3f s3m s3n p1 p2 p3f p3m p3n);
	foreach my $l (@languages) {
		print qq|<td valign="top">|;
		print qq|<table width="250" border="0" cellpadding="0" cellspacing="0">\n|;
		$entries{$l}->{conj} ||= {};
		my %con = %{ $entries{$l}->{conj} };
		foreach my $c(@codes) {
			$con{$c} ||= "";
			print qq|<tr>
<td>| . $conjcodes->{$c} . qq|</td>
<td><input type="text" name="entries{$l}{conj}{$c}" value="$con{$c}" size="20"></td>
</tr>\n|;
		}
		print qq|</table>\n|;
	} 
	print qq|</td>\n|;
	print qq|</tr>\n|;

	print qq|<tr>
	<td class="viewlabel">Level</td>
	<td><select name="entries{$languages[0]}{level}">|;
# 	foreach (sort { $levels{$a} cmp $levels{$b} } keys %levels) {
	foreach (@levels) {
		$entries{$languages[0]}->{level} ||= 21;
		my $sel = ($_->[0] == $entries{$languages[0]}->{level}) ? " selected " : "";
		print qq|\t<option value="$_->[0]" $sel> $_->[1]</option>\n|;
	}
	print qq|</select></td>
</tr>|;
	print qq|<tr>
	<td colspan="3" class="tableheader"><input type="submit" value="Speichern"></td>
</tr>	
<tr>
	<td colspan="3" align="right"><br>
	<a href="dict.pl?view=$id">Details</a>
	</td>
</tr>\n|;
	
	print qq|</table>\n|;
	print qq|</form>\n|;
}


sub checkinput {

	my $error;
	foreach (@languages) {
		$error .= "Wort in Sprache $langstrings{$_} muss ausgefuellt sein!<br>" unless $in{entries}->{$_}->{word};
	}


	my $stmt = "select id, langcode, word from Entries where (";
	foreach (@languages) {
		$stmt .= " (langcode = '$_' and word = " . $dbh->quote($in{entries}->{$_}->{word}) . ") or";
	}
	$stmt =~ s/or$/) /;
	$stmt .= " and id != $in{edit} " if ($in{edit} ne "new");

	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);
	my %words;
	foreach (@data) {
		push @{$words{$_->[1]}}, $_->[0];
	}
	my @l = @languages;
	my $l1 = shift (@l);
	foreach my $wid(@{$words{$l1}}) {
		my $cnt = -1;
		foreach my $l(@l) {
			$cnt++ if (grep /^$wid$/, @{$words{$l}});
		}
		if ($cnt == $#l) {
			$error .= "Dieser Eintrag existiert bereits (ID: $wid)<br>";
		}
	}
	return $error;
}

sub delentry {

	my $stmt = "delete from Entries where id = $in{del}";
	dprint $stmt;
	DB_insertdata(STMT => $stmt, DBH => $dbh);

	$stmt = "delete from Conjugations where entryid = $in{del}";
	dprint $stmt;
	DB_insertdata(STMT => $stmt, DBH => $dbh);

	$stmt = "delete from Entries_in_Lessions where entryid = $in{del}";
	dprint $stmt;
	DB_insertdata(STMT => $stmt, DBH => $dbh);

	$stmt = "delete from Entries_in_Levels where entryid = $in{del}";
	dprint $stmt;
	DB_insertdata(STMT => $stmt, DBH => $dbh);

	$stmt = "delete from User_Lessions_Entries where entryid = $in{del}";
	dprint $stmt;
	DB_insertdata(STMT => $stmt, DBH => $dbh);

	print qq|
<table border="0" cellpadding="0" cellspacing="0" width="600">
<tr>
	<td><b>Eintrag wurde gel&ouml;scht.</td>
</tr>
</table>|;	

}

sub saveentry {

	my $new = 0;
	if ($in{edit} eq "new") {
		$new = 1;
		my $idstmt = "select max(id) from Entries";
		my @iddata = DB_getdata(STMT => $idstmt, DBH => $dbh, ONECOLUMN => 1);
		if ($iddata[0]) {
			$in{edit} = $iddata[0]+1;
		} else {
			$in{edit} = 1;
		}
	}

	foreach my $l (@languages) {
		my $stmt;
		if ($new) {
			$stmt = "insert into Entries set id=$in{edit}, langcode='$l', ";
		} else {
			$stmt = "update Entries set ";
		}
		foreach my $e (qw(word article remark pronunciation antonyms synonyms example paraphrase entryusage)) {
			$stmt .= "$e = " . $dbh->quote(VocTrainer::Utils::to_unicode($in{entries}->{$l}->{$e} || "")) . ", ";
		}
		if (!$new) {
			$stmt =~ s/, $//;
			$stmt .= " where id=$in{edit} and langcode='$l'";
		} else {
			$stmt .= " createdate = NOW() ";
		}
		# dprint($stmt);
		DB_insertdata(STMT => $stmt, DBH => $dbh);
		if (!$new) {
			my $del = "delete from Conjugations where entryid=$in{edit} and langcode = '$l'";
			# dprint $del;
			DB_insertdata(STMT => $del, DBH => $dbh);
		}
		if ($in{entries}->{$l}->{conj}) {
			my %conj = %{ $in{entries}->{$l}->{conj} } ;
			foreach my $c (keys %conj) {
				if ($conj{$c}) {
					my $ins = "insert into Conjugations values ($in{edit}, '$l', '$c', " . $dbh->quote($conj{$c}) . ")";
					# dprint($ins);
					DB_insertdata(STMT => $ins, DBH => $dbh);
				}
			}
		}
	}

	my $del = "delete from Entries_in_Levels where entryid = $in{edit}";
	DB_insertdata(STMT => $del, DBH => $dbh);
	my $ins = "insert into Entries_in_Levels values ($in{entries}->{$languages[0]}->{level}, $in{edit})";
	DB_insertdata(STMT => $ins, DBH => $dbh);

	1;
}


sub get_pronouns {
	my %args = (DBH => '',
				@_);

	die "No DBH" unless $args{DBH};

	my $stmt = "select conjugationcode, langcode, pronoun from Pronouns";
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);	
	my %pronouns;
	foreach (@data) {
		$pronouns{$_->[0]}->{$_->[1]} = VocTrainer::Utils::unicode2html($_->[2]);
	}
	return \%pronouns;
}

sub get_conjcodes {
	my %args = (DBH => '',
				@_);

	die "No DBH" unless $args{DBH};

	my $stmt = "select code, description from Conjugationcodes";
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);	
	my %conjcodes;
	foreach (@data) {
		$conjcodes{$_->[0]} = VocTrainer::Utils::unicode2html($_->[1]);
	}
	return \%conjcodes;
}


sub searchform {

#	my $langselect = qq|<select name="lang">\n|;
#	foreach (@languages) {
#		my $sel = ($_ eq $in{lang}) ? "selected" : "";
#		$langselect .= qq|<option value="$_" $sel>$langstrings{$_}</option>\n|;
#	}
#	$langselect .= qq|</select>\n|;
	print qq|<form method="POST" action="dict.pl">
<input type="hidden" name="edit" value="">
<input type="hidden" name="view" value="">
<table width="$w" border="0" cellpadding="2" cellspacing="0">
<tr>
	<td width="500">
		<input type="text" name="q" value="$in{q}">
		<input type="submit" name="action_search" value="Suchen">
	</td>
	<td align="right" width="100">
|;
	if ($user->{isadmin}) {
		print qq|<a href="dict.pl?edit=new">Neuer Eintrag</a>|;
	}

	print qq|</td>
</tr>
<tr>
	<td>Es kann nur nach einem Wort gesucht werden. Ein Stern '<b>*</b>' kann als Platzhalter verwendet werden (z.b. findet 'tam*' sowohl 'tampoco' als auch 'tambi&eacute;n').</td>
</tr>
</table>\n</form>\n|;
}

sub print_list {
	my $entries = shift;
	my $user = shift || {};
	my %entries = %{$entries};

	return unless keys %entries;

	my $tdw = ($user->{isadmin}) ? 260 : 280;
	my $cols = ($user->{isadmin}) ? 4 : 3;

	print qq|<table border="0" cellspacing="0" cellpadding="3" width="$w">\n|;
	print qq|<tr class="tableheader">\n|;
	# print qq|<td class="tableheader">ID</td>\n|;
	foreach (@languages) {
		my $desc = ($_ eq $in{sortlang} && $in{desc} == 0 ) ? 1 : 0;
		print qq|<td class="tableheader"><a href="dict.pl?q=$in{q};sortlang=$_;desc=$desc" class="tableheader">$langstrings{$_}</a></td>\n|;
	}
	print qq|<td class="tableheader">&nbsp;</td>|;
	if ($user->{isadmin}) {
		print qq|<td class="tableheader">&nbsp;</td>|;
	}
	print qq|</tr>\n|;

	my $sort;
	if ($in{desc}) {
		$sort = sub { uc($entries{$b}->{$in{sortlang}}->{word}) cmp uc($entries{$a}->{$in{sortlang}}->{word}) };
	} else {
		$sort = sub { uc($entries{$a}->{$in{sortlang}}->{word}) cmp uc($entries{$b}->{$in{sortlang}}->{word}) };
	}
	my $i = 0;
	foreach my $id (sort $sort keys %entries) {
		my $row = "row" . ($i % 2);
		print qq|<tr class="$row">\n|;
		# print qq|<td width="20">$id</td>|;
		foreach my $lang (@languages) {
			print qq|\t<td width="$tdw" valign="top">|;
			if ($entries{$id}->{$lang}->{article}) {
				print $entries{$id}->{$lang}->{article} . " ";
			}
			print $entries{$id}->{$lang}->{word};
			print " (Synonyme: " . $entries{$id}->{$lang}->{synonyms} . ")" if $entries{$id}->{$lang}->{synonyms};
			print qq|</td>\n|;
		}
		print qq|\t<td width="40" align="center"><a href="dict.pl?view=$id;q=$in{q};sortlang=$in{sortlang};desc=$in{desc}">Details</a></td>\n|;
		if ($user->{isadmin}) {
			print qq|\t<td width="40" align="center">
		<a href="javascript:askdel('$id');">del</a>
		&nbsp;
		<a href="dict.pl?edit=$id">edit</a>
	</td>\n|;
		}
		print qq|</tr>\n|;
		$i++;
	}

	print qq|</table>|;
}

sub get_entries {

	# my $stmt = "select id, langcode, word, typeid, remark, pronunciation, antonyms, synonyms, example, paraphrase, entryusage from Entries";
	my $stmt = "select id, langcode, word, article, synonyms from Entries where ";

	$in{q} =~ s/^\s+//;
	$in{q} =~ s/\s+$//;
#	if ($in{q}) {
		my $w = $in{q};

		$w =~ s/\*/%/g;	
		$stmt .= " word like " . $dbh->quote(VocTrainer::Utils::to_unicode($w)); #   . " and langcode = '$in{lang}'";
		$stmt .= " or synonyms like " . $dbh->quote("%" . VocTrainer::Utils::to_unicode($w) . "%"); #   . " and langcode = '$in{lang}'";
#	}

	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);

	my %entries;

	foreach (@data) {
		$entries{$_->[0]}->{$_->[1]} = {
			langcode => $_->[1],
			word => $_->[2],
			article => $_->[3],
			synonyms => $_->[4],
		}; 
	} 

	my $eids = join(",", keys %entries);

	if ($eids) {
		if ($in{q}) {
			# my $stmt = "select id, langcode, word, typeid, remark, pronunciation, antonyms, synonyms, example, paraphrase, entryusage from Entries";
			my $stmt = "select id, langcode, word, article, synonyms from Entries";
			$stmt .= " where id in ($eids) "; #  and langcode != '$in{lang}'";
			my @data = DB_getdata(STMT => $stmt, DBH => $dbh);
			foreach (@data) {
				$entries{$_->[0]}->{$_->[1]} = {
					langcode => $_->[1],
					word => $_->[2],
					article => $_->[3],
					synonyms => $_->[4],
				}; 
			} 
		}


#		my $cstmt = "select entryid, langcode, conjugationcode, conjugation from Conjugations where entryid in ($eids)";
#		my @cdata = DB_getdata(STMT => $cstmt, DBH => $dbh);
#
#		foreach (@cdata) {
#			# $entries{$_->[0]}->{$_->[1]}->{conjugations} = {} unless $entries{$_->[0]}->{conjugations};
#			$entries{$_->[0]}->{$_->[1]}->{conjugations}->{$_->[2]} = $_->[3];
#		}
	}
	
	if (%entries) {
		return VocTrainer::Utils::unicode2html(\%entries);
	} else {
		return 0;
	}
}


sub get_levels {
	my $stmt = "select id, name from Levels order by seqnr";
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);
	return @data;
#	my %l = map { $_->[0] => $_->[1] } @data;
#	return %l;
}
