#!/usr/bin/perl -w

use strict;

use CGI;
use locale;
use POSIX qw(locale_h setlocale LC_ALL LC_CTYPE);

use VocTrainer::Database;
use VocTrainer::Layout;
use VocTrainer::Users;
use VocTrainer::Params;
use VocTrainer::Utils;

use constant NOLEVEL => 21;   # id of "No Level" - Level
# TODO get from db
my @languages = ('de', 'es');
my %langstrings = (
    "de" => "Deutsch",
    "es" => "Espaol"
);

my $q = new CGI;
my %in = GetParams(QUERY => $q);

my $dbh = DB_open();

my %levels;

my $user = VocTrainer::Users::CheckUser(DBH => $dbh);

$in{level} ||= $user->{level};

if (!$user->{uid}) {
	print $q->header();
	&VocTrainer::Layout::printhtmlheader(TITLE => "&Uuml;bungen" , USER => $user);
	#print $q->redirect("./login.pl?back=lession.pl");
	print qq|Bitte zuerst <a href="login.pl?back=lession.pl"> einloggen</a>|;
} else {
	print $q->header();
	&VocTrainer::Layout::printhtmlheader(TITLE => "&Uuml;bungen" , USER => $user);
	%levels = &get_levels;

	if ($in{action}) {
		my $todo = $in{action}[0];
		# dprint $todo;
		if ($todo eq "resolve") {
			my @resolved = &resolve(WORDS => $in{words});
			&formout(WORDS => $in{words}, RESOLVED => \@resolved, ADDEDPOINTS => 0);
		}
		elsif ($todo eq "addpoints") {
			my $addp = 0;
			foreach (@{$in{addpoint}}) {
				$addp++ if ($_);
			}
			&update_addpoints(WORDS => $in{words}, POINTS => $in{addpoint});
			&formout(WORDS => $in{words}, RESOLVED => $in{resolved}, ADDEDPOINTS => $addp);
		}

		elsif ($todo eq "restart") {
			$in{answer} = [];
			$in{usersessionid} = undef;
			$in{words} = &get_words_from_lession unless $in{words};
			&save_lession(WORDS => $in{words}, USER => $user);
			&formout(WORDS => $in{words});
		}
		elsif ($todo eq "newlession") {
			$in{answer} = [];
			my $random = &get_random_entries(10);
			&save_lession(WORDS => $random, USER => $user);
			&formout(WORDS => $random);
		} else {
			&statistics;
		}
	} else {
#		my $random = &get_random_entries(10);
#		&save_lession(WORDS => $random, USER => $user);
#		&formout(WORDS => $random);
		&statistics;
	}

}
DB_close(DBH => $dbh);

sub statistics {

	my $stmt = "select l.id, l.name, l.level, u.lessionid, u.lessiongrade, u.lessiondate from User_Lessions u, Lessions l where l.id = u.lessionid and u.userid = $user->{uid}";
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);
	my $lessions = $#data + 1;
	my %tmp = map { $_->[0] => 1 } @data;
	my $userlessions = keys %tmp;


	my %lessions;
	my %userlessions;
	my $totalgrade;
	my %cnts;
	my %grades;
	my %levelgrades; my %levelcnts;
	my $mind = 99999999999999;
	my $maxd = 00000000000000;
	foreach (@data) {
		$_->[2] ||= NOLEVEL;
		$lessions{$_->[0]} = { name => $_->[1], level => $_->[2], timestamp => $_->[5] };
		$cnts{$_->[0]}++;
		$userlessions{$_->[3]} = { id => $_->[0], grade => $_->[4], timestamp => $_->[5] };
		$totalgrade += $_->[4];
		$grades{$_->[0]} += $_->[4];
		$levelgrades{$_->[2]} += $_->[4];
		$levelcnts{$_->[2]}++;
		$mind = $_->[5] if ($mind > $_->[5]);
		$maxd = $_->[5] if ($maxd < $_->[5]);
	}
	my $mindate = substr($mind, 6,2) . "." . substr($mind, 4,2) . "." .substr($mind, 0,4);
	my $maxdate = substr($maxd, 6,2) . "." . substr($maxd, 4,2) . "." .substr($maxd, 0,4);
	
	my $avggrade = ($lessions == 0) ? 0 : sprintf("%.2f", ($totalgrade / $lessions));
	foreach (keys %lessions) {
		$lessions{$_}->{avggrade} = sprintf("%.2f", ($grades{$_} / $cnts{$_}) );
	}

	#dprint (\%lessions);

	my $pastlessions = "";
	my $i = 0;
	foreach (sort { $lessions{$b}->{timestamp} <=> $lessions{$a}->{timestamp} }  keys %lessions) {
		$pastlessions .= "<option value='$_'> $lessions{$_}->{name} ($lessions{$_}->{avggrade} %)</option>\n";
	}

#	$stmt = "select count(id) from User_Lessions where userid = $user->{uid}";
#	@data = DB_getdata(STMT => $stmt, DBH => $dbh, ONECOLUMN => 1);
#	my $userlessions = $data[0];

	my $avg_per_lession = ($lessions == 0) ? 0 : sprintf("%.2f", ($lessions / $userlessions));

#	$stmt = 'select avg(lessiongrade) from User_Lessions where userid = ' . $user->{uid};
#	@data = DB_getdata(STMT => $stmt, DBH => $dbh, ONECOLUMN => 1);
#	my $avggrade = sprintf("%.2f", $data[0] || 0);

#	$stmt = 'select date_format(min(lessiondate), "%d.%m.%Y") from User_Lessions where userid = ' . $user->{uid};
#	@data = DB_getdata(STMT => $stmt, DBH => $dbh, ONECOLUMN => 1);
#	my $mindate = $data[0];

#	$stmt = 'select date_format(max(lessiondate), "%d.%m.%Y") from User_Lessions where userid = ' . $user->{uid};
#	@data = DB_getdata(STMT => $stmt, DBH => $dbh, ONECOLUMN => 1);
#	my $maxdate = $data[0];

#	$stmt = 'select distinct l.id, l.name, avg(ul.lessiongrade) from Lessions l, User_Lessions ul where ul.lessionid = l.id and ul.userid = ' . $user->{uid} . " group by l.id order by createdate desc ";
#	@data = DB_getdata(STMT => $stmt, DBH => $dbh);
#	my $pastlessions = "";
#	foreach (@data) {
#		$pastlessions .= "<option value='$_->[0]'> $_->[1] (" . sprintf("%.2f", $_->[2]) . " %)</option>\n";
#	}

	$stmt = "select u.id, u.name, count(l.id), avg(l.lessiongrade), count(l.id) * avg(l.lessiongrade) as mygrade, u.level from Users u left join User_Lessions l on l.userid = u.id group by u.id"; #  having mygrade is not null"; #  order by mygrade desc limit 5";	
#	$stmt = "select u.id, u.name, avg(l.lessiongrade) as mygrade from Users u left join User_Lessions l on l.userid = u.id group by u.id having mygrade is not null order by mygrade desc limit 5";
	my @users = DB_getdata(STMT => $stmt, DBH => $dbh);
	foreach (@users) {
		$_->[2] ||= 0;
		$_->[3] ||= 0;
	}

print qq|

<table border="0" cellpadding="3" cellspacing="0" width="600">
<tr>
	<td width="400" valign="top">
	<div class="subtitle">Statisik</div>
	Du hast bisher <b>$userlessions</b> verschiedene &Uuml;bungen durchgef&uuml;hrt.
	Jede der &Uuml;bungen hast du im Durchschnitt <b>$avg_per_lession</b> mal wiederholt. 
	Insgesamt hast du bereits <b>$lessions</b> &Uuml;bungen gemacht.<br>
	Im Durchschnitt hast du dabei <b>$avggrade</b> % richtig gehabt.<br>
	|;
	print qq|Die erste &Uuml;bung hast du am $mindate und die letzte am $maxdate gemacht.<br>| if ($mindate && $mindate ne "99.99.9999");
	print qq|<br>|;
	if (keys %levelgrades) {

		print qq|
<table border="0" cellpadding="3" cellspacing="0" width="100%">
<tr>
	<td colspan="2" class="subtitle">Ergebnisse nach Level</td>
</tr>
<tr>
	<td class="tableheader">Level</td>
	<td class="tableheader" align="right">Anzahl</td>
	<td class="tableheader" align="right">Ergebnis</td>
</tr>|;
		my $i = 0;
		foreach (sort { $levels{$a}->{seqnr} <=> $levels{$b}->{seqnr} } keys %levelgrades) {
			print qq|<tr class="row| . ($i%2) . qq|">|;
			print qq|<td>| . $levels{$_}->{name} . qq|</td>|;
			print qq|<td align="right">$levelcnts{$_} x&nbsp;</td>|;
			print qq|<td align="right">| . sprintf("%.2f", ( $levelgrades{$_} / $levelcnts{$_}) ) . qq| %</td>|;
			print qq|</tr>|;
			$i++;
		}

		print qq|<tr>
	<td class="tableheader">Gesamtwertung:</td>
	<td class="tableheader" align="right">$lessions x&nbsp;</td>
	<td class="tableheader" align="right">$avggrade %</td>
</tr>|;

		print qq|</table><br>|;
	}
	print qq|Dein aktueller Level ist: <b>| . $levels{$user->{level}}->{name} . qq|</b> [<a href="register.pl">&Auml;ndern</a>]<br><br>|;
	print qq|<br>

	<form>
		<div class="subtitle">Neue &Uuml;bung</div>
		<select name="level">
|;
	foreach (sort { $levels{$a}->{seqnr} <=> $levels{$b}->{seqnr} } keys %levels) {
		next unless $levels{$_}->{status} eq "F";
		my $sel = ($_ == $user->{level}) ? "selected" : ""; 
		print qq|<option value="$_" $sel>| . $levels{$_}->{name} . qq|</option>\n|;
	}
	print qq|
		</select>
		<input type="submit" name="action_newlession" value="Neue &Uuml;bung">
	</form>
|;

	if ($pastlessions) {
		print qq|
	<form><br>
		<div class="subtitle">&Uuml;bung wiederholen</div>
		Du kannst auch bereits durchgef&uuml;hrte &Uuml;bungen wiederholen.<br>
		<select name="lessionid">
			$pastlessions
		</select><br>
		<input type="submit" name="action_restart" value="&Uuml;bung wiederholen">
	</form>
|;
	}

	print qq|
	</td>
	<td valign="top" width="200" align="right">
	<table border="0" cellpadding="2" cellspacing="0" width="190">
	<tr>
		<td colspan="4" align="center" class="subtitle">Die Besten nach Ergebnis</td>
	</tr>
	<tr>
		<td class="tableheader" align="center">#</td>
		<td class="tableheader">Name</td>
		<td class="tableheader" align="center" TITLE="Anzahl durchgef&uuml;hrter &Uuml;bungen">Cnt.</td>
		<td class="tableheader" align="center" TITLE="Durchschnitt richtiger L&ouml;sungen">%</td>
	</tr>
|;
	my $n = 1;
	foreach (sort { $b->[3] <=> $a->[3] } @users) {
		my $class = ($_->[0] == $user->{uid}) ? "row0" : "row1";
		print qq|	<tr class="$class">
		<td align="right" width="20">$n. &nbsp;</td>
		<td width="90">$_->[1]</td>
		<td align="right" width="30">| . $_->[2] . qq| x</td>
		<td align="right" width="50">| . sprintf("%.2f", $_->[3]) . qq| %</td>
	</tr>
|;
		last if $n == 5;
		$n++;
	}

	print qq|
	<tr>
		<td colspan="4" align="center" class="subtitle"><br>Die Fleissigsten</td>
	</tr>
	<tr>
		<td class="tableheader" align="center">#</td>
		<td class="tableheader">Name</td>
		<td class="tableheader" align="center" TITLE="Anzahl durchgef&uuml;hrter &Uuml;bungen">Cnt.</td>
		<td class="tableheader" align="center" TITLE="Durchschnitt richtiger L&ouml;sungen">%</td>
	</tr>
|;
	$n = 1;
	foreach (sort { $b->[2] <=> $a->[2] } @users) {
		my $class = ($_->[0] == $user->{uid}) ? "row0" : "row1";
		print qq|	<tr class="$class">
		<td align="right" width="20">$n. &nbsp;</td>
		<td width="90">$_->[1]</td>
		<td align="right" width="30">| . $_->[2] . qq| x</td>
		<td align="right" width="50">| . sprintf("%.2f", $_->[3]) . qq| %</td>
	</tr>
|;
		last if $n == 5;
		$n++;
	}

	print qq|
</tr>
</table>
	|;
		
}

sub resolve {
	my %args = (WORDS => [],
				@_);

	my $stmt = "select id, word, article, synonyms from Entries where ";
	foreach (@{$args{WORDS}}) {
		$stmt .= " ( id = $_->{id} and langcode != '$_->{langcode}' ) or ";
	}
	$stmt =~ s/or $//;
	# dprint $stmt;
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);
	# dprint (\@data);
	my %an =  map { $_->[0] => { word => $_->[1], article => $_->[2], synonyms => $_->[3] } } @data;
	my $answers = VocTrainer::Utils::unicode2ascii(\%an); 
	# dprint ($answers);

	my @resolved;
	my $i = 0;
	my @correctids;
	foreach (@{$args{WORDS}}) {
		my $id = $_->{id};
#	 	print "ID: $id $answers->{$id}->{word}<br>";
		
		(my $useranswer = uc($in{answer}[$i])) =~ s/\s+/ /g;
		$useranswer =~ s/^\s+//;
		$useranswer =~ s/\s+$//;
#		print "User: $useranswer<br>";
#		if ($in{answer}[$i]
		my $word = uc($answers->{$id}->{word});
		my $art = uc($answers->{$id}->{article});
		my $syn = uc($answers->{$id}->{synonyms});
		if ($useranswer eq $word or $useranswer eq "$art $word") {
#			print "Korrekt!<br>";
			push @resolved, 1;
			push @correctids, $id;
		} elsif ($syn && $syn =~ /$useranswer/) {
			push @resolved, 1;
			push @correctids, $id;
		} else {
#			print "Falsch!<br>";
			my $correct = "";
			$correct = "$answers->{$id}->{article} " if ($answers->{$id}->{article});
			$correct .= $answers->{$id}->{word};
			$correct .= " (Synonyme: " . $answers->{$id}->{synonyms} . ")" if $syn;
			push @resolved, $correct;
		}
		$i++;
	}

	my $cids = join(",", @correctids);
	if ($cids) {
		my $update = "update User_Lessions_Entries set correct = 1 where userlessionid = $in{userlessionid} and entryid in ($cids)";
		DB_insertdata(STMT => $update, DBH => $dbh);
	}
	return @resolved;	
}

sub update_addpoints {
	my %args = (WORDS => [],
				POINTS => [],
				@_);

	my @points = @{$args{POINTS}};

	my @add = ();
	foreach my $i (0..$#points) {
		if ($points[$i]) {
			push @add, $args{WORDS}->[$i]->{id};
		}
	}

	my $del = "update User_Lessions_Entries set addpoint = 0 where userlessionid = $in{userlessionid}";
	DB_insertdata(STMT => $del, DBH => $dbh);
	my $ids = join(",", @add);
	if ($ids) {
		my $update = "update User_Lessions_Entries set addpoint = 1 where userlessionid = $in{userlessionid} and entryid in ($ids)";
		DB_insertdata(STMT => $update, DBH => $dbh);
	}
}

sub formout {
	my %args = (WORDS => [],
				RESOLVED => '',
				ADDEDPOINTS => 0,
				@_);


	my @words = @{$args{WORDS}};

	my @resolved;
	my $tdw = 300;
	if ($args{RESOLVED}) {
		@resolved = @{$args{RESOLVED}};
		$tdw=280;
	}
	print qq|<form method="POST">

<input type="hidden" name="lessionid" value="$in{lessionid}">
<input type="hidden" name="userlessionid" value="$in{userlessionid}">

<table border="0" cellspacing="0" cellpadding="3" width="600">
<tr class="tableheader">	
	<td class="tableheader">$langstrings{$languages[0]}</td>
	<td class="tableheader">$langstrings{$languages[1]}</td>|;

	if (@resolved) {
		print qq|<td class="tableheader">&nbsp;</td>|;
		print qq|<td class="tableheader">&nbsp;</td>|;
	}
	print qq|</tr>|;
	my $i = 0;
	my $false;
	foreach my $w(@words) {
		my $row = "row" . $i % 2;
		print qq|<tr class="$row">\n|;
		print qq|\t<input type="hidden" name="words[$i]{id}" value="$w->{id}">\n|;

		foreach my $l (@languages) {
			print qq|\t<td valign="top" width="$tdw">\n\t|;
			if ($w->{langcode} eq $l) {
				print $w->{article} . " " if $w->{article};
				print $w->{word};
				print qq|\t<input type="hidden" name="words[$i]{word}" value="$w->{word}">\n|;
				print qq|\t<input type="hidden" name="words[$i]{article}" value="$w->{article}">\n|;
				print qq|\t<input type="hidden" name="words[$i]{langcode}" value="$w->{langcode}">\n|;
			} else {
				my $answer = $in{answer}[$i] || "";
				if (@resolved) {
					print qq|<input type="hidden" name="answer[$i]" value="$answer">\n|;
					print qq|<input type="hidden" name="resolved[$i]" value="$resolved[$i]">\n|;
					my $style = ($resolved[$i] eq "1" ) ? "correct" : "false";
					print qq|\t<span class="$style">$answer</span>\n|;	
					if ($resolved[$i] ne "1") {
						print qq|\n<br><b>$resolved[$i]</b>|;
						$false++;
					}
				} else {
					print qq|<input type="text" name="answer[$i]" value="$answer" size="30">\n|;
				}
			}
			print qq|</td>\n|;
		}
		if (@resolved) {
			my $checked = ($in{addpoint}[$i]) ? "checked" : "";
			print qq|<td width="40"><a href="javascript:details('$w->{id}')">Details</a></td>|;
			print qq|<td width="40">|;
			if ($resolved[$i] ne "1") {
				print qq|<input type="checkbox" name="addpoint[$i]" $checked>|;
			} else {
				print qq|&nbsp;|;
				print qq|<input type="hidden" name="addpoint[$i]" value="0">|;
			}
			print qq|</td>|;
		}
		print qq|</tr>\n|;
		$i++;
	}	

	if (@resolved) {
		print qq|<tr><td colspan="3">|;
		my $cnt = $#resolved + 1;
		$false ||= 0;
		my $correct = $cnt - $false;
		my $perc = $correct / $cnt * 100;
		
		print qq|<table width="100%"><tr>
<td valign="top"><span class="correct">$correct</span> von <b>$cnt</b> W&ouml;rtern <span class="correct">RICHTIG</span> gel&ouml;st. Das sind $perc %<br>
|;

		my $update;
		if ($args{ADDEDPOINTS}) {
			my $perc2 = ($correct + $args{ADDEDPOINTS}) / $cnt * 100;
			print qq|$args{ADDEDPOINTS} Punkt(e) wurde(n) hinzugef&uuml;gt, das ergibt gesamt | . ($correct + $args{ADDEDPOINTS}) . qq| von $cnt Punkten ($perc2 %)<br>|;
			$update = "update User_Lessions set lessiongrade = '$perc2' where id = $in{userlessionid}";
		} else {
			$update = "update User_Lessions set lessiongrade = '$perc' where id = $in{userlessionid}";
		}
		DB_insertdata(STMT => $update, DBH => $dbh);

		print qq|
<br>
<input type="submit" name="action_newlession" value="Neue Aufgabe">
<input type="submit" name="action_restart" value="Aufgabe wiederholen">
<input type="submit" name="action_statistics" value="Statistik">
</td>
<td valign="top" align="right">
	<input type="submit" name="action_addpoints" value="Punkte &auml;ndern">
</td></tr></table>
</td>
</tr>\n|;
	} else {
		print qq|<tr><td colspan="2" class="tableheader">|;
		print qq|<input type="submit" name="action_resolve" value="Aufl&ouml;sen">|;
		print qq|</td></tr>|;
	}
	print qq|</table>|;
	print qq|</form>|;
}


sub save_lession {
	my %args = (WORDS => [], 
				USER => {},
				@_);

	# dprint($args{USER});
	my $new = 0;
	if ($in{lessionid}) {


	} else {
		$new = 1;
		my @lt = localtime();
		$lt[4]++; $lt[5]+=1900;
		my $title = $args{USER}->{username} . " ";
		if ($in{level} != NOLEVEL) {
			$title .= "(" . $levels{$in{level}}->{name} . ") ";
		}
		$title .= join(".", map { sprintf("%02d", $_) } @lt[3,4]);
		$title .= ".$lt[5] ";
		$title .= join(":", map { sprintf("%02d", $_) } @lt[2,1]);

		# dprint ("New Lession: $title");


		my $ins = "insert into Lessions set " . 
			" name = " . $dbh->quote($title) . ", " . 
			" createdate = NOW()";
		$ins .= ", level = " . $in{level} if ($in{level} != NOLEVEL);
		# dprint ($ins);
		DB_insertdata(STMT => $ins, DBH => $dbh);

		$in{lessionid} = &DB_getlastinsertid(DBH => $dbh);

		# dprint ("Lession ID: $in{lessionid}");
	}

	my $stmt = "insert into User_Lessions values ('', $user->{uid}, $in{lessionid}, 0.0, NOW())";
	# dprint ($stmt);
	DB_insertdata(STMT => $stmt, DBH => $dbh);

	$in{userlessionid} = &DB_getlastinsertid(DBH => $dbh);
	# dprint ("User Lession ID: $in{userlessionid}");

	my @words = @{$args{WORDS}};

	foreach my $w (@words) {
		if ($new == 1) {
			my $stmt = "insert into Entries_in_Lessions values ($in{lessionid}, $w->{id}, '$w->{langcode}')";
			DB_insertdata(STMT => $stmt, DBH => $dbh);
		}

		my $stmt = "insert into User_Lessions_Entries values ($in{userlessionid}, $w->{id}, '$w->{langcode}', 0, 0)";
		DB_insertdata(STMT => $stmt, DBH => $dbh);

	}

}

sub get_words_from_lession {

	my $stmt = "select l.entryid, l.entrylangcode, e.word, e.article from Entries_in_Lessions l, Entries e " . 
		" where l.lessionid = $in{lessionid} and l.entryid = e.id and l.entrylangcode = e.langcode";
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);

	my @words;
	foreach (@data) {
		push @words, {
			id => $_->[0],
			langcode => $_->[1],
			word => $_->[2],
			article => $_->[3],
		};
	}

	return VocTrainer::Utils::unicode2html(\@words);

}
sub get_random_entries {
	my $cnt = shift;
	my $ids = shift;

	my $stmt;
	if ($in{level} && $in{level} != NOLEVEL) {
		$stmt = "select distinct entryid from Entries_in_Levels where levelid != " . NOLEVEL . " and levelid = " . $in{level};
		$stmt .= " and entryid not in ($ids) " if $ids;
	} else {
		my $stmt = "select distinct id from Entries ";
		$stmt .= " where id not in ($ids) " if $ids;
	}
	$stmt .= " order by RAND() limit $cnt";
	# dprint $stmt;
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh, ONECOLUMN => 1);

	$stmt = "select id, langcode, word, article from Entries where ";
	foreach (@data) {
		my $rand = int(rand() + .5);
		$stmt .= "( id = $_ and langcode='$languages[$rand]' ) or ";
	}
	$stmt =~ s/or $//;
	# dprint $stmt;

	@data = DB_getdata(STMT => $stmt, DBH => $dbh);
	my @random;
	foreach (@data) {
		push @random, {
			id => $_->[0],
			langcode => $_->[1],
			word => $_->[2],
			article => $_->[3],
		};
	}

	return VocTrainer::Utils::unicode2html(\@random);
}


sub get_levels {
	my $stmt = "select id, name, seqnr, status from Levels order by seqnr";
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);
	my %levels = map { $_->[0] => { name => $_->[1], seqnr => $_->[2], status => $_->[3] } } @data;
	return %levels;
}
