#!/usr/bin/perl -w

use strict;

use CGI;
use VocTrainer::Database;
use VocTrainer::Layout;
use VocTrainer::Params;
use VocTrainer::Users;

my $mailprog = "/usr/sbin/sendmail -f 'voctrainer\@weirdoz.net'";

my $q = new CGI;
my %in = GetParams (QUERY => $q);
my $dbh = DB_open();

print $q->header;

my $user = VocTrainer::Users::CheckUser(DBH => $dbh);

my $title = ($user->{uid}) ? "Einstellungen" : "Register";

&VocTrainer::Layout::printhtmlheader(TITLE => $title, USER => $user);

if ($user->{uid}) {
	if ($in{action}) {
		my $todo = $in{action}[0];
		if ($todo eq "register") {
			my $error = &check_input();
			if ($error) {
				&formout(ERROR => $error);
			} else {
				&update_ok;
			}
		} else {
			&formout;
		}
	} else {
		&getuser;
		&formout;
	}
} else {

	if ($in{action}) {
		my $todo = $in{action}[0];
		if ($todo eq "register") {
			my $error = &check_input();
			if ($error) {
				&formout(ERROR => $error);
			} else {
				&register_ok;
			}
		} else {
			&formout;
		}
	} else {
		$in{level} ||= 1;
		$in{reminder} = 1;
		&formout;
	}
}

DB_close(DBH => $dbh);

sub register_ok {

	my @chars = ( "a" .. "z", "A" .. "Z", 0 .. 9 );
	$in{password} = join ("", @chars[ map { rand @chars } (1..8) ]);

	my $ins = "insert into Users set " .
		" name = " . $dbh->quote($in{name}) . ", " . 
		" password = " . $dbh->quote($in{password}) . ", " . 
		" firstname = " . $dbh->quote($in{firstname}) . ", " . 
		" lastname = " . $dbh->quote($in{lastname}) . ", " .
		" level = " . $dbh->quote($in{level}) . ", " .
		" email = " . $dbh->quote($in{email});
	$ins .= ", reminder = 1 " if $in{reminder};

	DB_insertdata(STMT => $ins, DBH => $dbh);

	&sendok();

	print qq|
<table border="0" cellpadding="3" cellspacing="0" width="600">
<tr>
	<td>
		<b>Registrierung war erfolgreich.</b><br>
		Du bekommst in K&uuml;rze dein Passwort zugesandt.<br>
	</td>
</tr>
</table>
|;
}

sub sendok {

	my $rem = ($in{reminder}) ? "Ja" : "Nein";

	my $mailtxt = qq|From: VocTrainer <voctrainer\@weirdoz.net>
Reply-to: voctrainer\@weirdoz.net
To: $in{firstname} $in{lastname} <$in{email}>
Subject: VocTrainer Registrierung

Du hast dich vom Rechner $ENV{REMOTE_ADDR} beim WeirdWeb VocTrainer
angemeldet. Du kannst jetzt die bungen durchfhren.

Deine Daten:
Username: $in{name}
Passwort: $in{password}
Name:     $in{firstname} $in{lastname}
Reminder: $rem


--
http://www.weirdoz.net/voctrainer
voctrainer\@weirdoz.net
|;

    open (MAIL, "|$mailprog $in{email}") || die "Can't open $mailprog $!";
    print MAIL $mailtxt;
    close (MAIL);


}

sub update_ok {

	my $upd = "update Users set " .
		" firstname = " . $dbh->quote($in{firstname}) . ", " . 
		" lastname = " . $dbh->quote($in{lastname}) . ", " .
		" email = " . $dbh->quote($in{email}) . ", " . 
		" level = '$in{level}'";
	$upd .= ", password = " . $dbh->quote($in{passwd1}) if ($in{passwd1});
	$upd .= ", reminder = " . ( ($in{reminder}) ? 1 : 0 ) . " ";
	$upd .= " where id = " . $user->{uid};
#	print STDERR "$upd\n";
	DB_insertdata(STMT => $upd, DBH => $dbh);

#	&sendok();

	print qq|
<table border="0" cellpadding="3" cellspacing="0" width="600">
<tr>
	<td>
		<b>&Auml;nderungen deiner Daten waren erfolgreich.</b><br>
	</td>
</tr>
</table>
|;


}
sub check_input {
	my $error;
	my %req = (
		name => "Username",
		firstname => "Vorname",
		lastname => "Nachname",
		email => "Email",
	);
	my @reqfields;
	foreach (keys %req) {
		push @reqfields, $req{$_} unless $in{$_};
	}
	if (@reqfields) {
		$error .= "Folgende Felder m&uuml;ssen ausgef&uuml;llt sein: <b>" . join (", ", @reqfields) . "</b>.<br>";
	}

	if ($in{email}) {
		if ($in{email} !~  /^[^\s]+\@[\w\d\.\-\_]+\.[\w]{2,4}$/) {
			$error .= "Bitte eine g&uuml;ltige Email-Adresse angeben.<br>";
		}
	}

	if (!$user->{uid} && $in{name}) {
		my $stmt = "select id from Users where name = " . $dbh->quote($in{name});
		my @data = DB_getdata(STMT => $stmt, DBH => $dbh, ONECOLUM => 1);
		if ($data[0]) {
			$error .= "Dieser Username ist bereits verwendet, bitte einen anderen w&auml;hlen<br>";
		}
	}


	if ($user->{uid} && ($in{passwd1} || $in{passwd2}) ) {
		$error .= "Passw&ouml;rter stimmen nicht &uuml;berein" unless ($in{passwd1} eq $in{passwd2});
	}
	return $error;
}

sub getuser {
	my @cols = qw(firstname lastname email reminder level); 
	my $stmt = "select " . join(", ", @cols) . " from Users where id = " . $user->{uid};
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);
	foreach my $i(0..$#cols) {
		$in{$cols[$i]} = $data[0][$i];
	}
}

sub formout {
	my %args = (ERROR => '',
				@_);
	foreach (qw(name firstname lastname email reminder level)) {
		$in{$_} ||= "";
	}

	my $sel1 = ($in{reminder}) ? "checked" : "";
	my $sel0 = ($sel1) ? "": "checked";

	my $submit = ($user->{uid}) ? "Speichern" : "Register";	
	print qq|
<table border="0" cellpadding="3" cellspacing="0" width="600">
|;
	if ($args{ERROR}) {
		print qq|<tr>
<td class="error">$args{ERROR}</td>
</tr>|;
	}

	print qq|
<tr>
	<td>

<form method="POST">
<table border="0" cellpadding="3" cellspacing="0">
<tr>
	<td>Username</td>
	<td>|; 

	if ($user->{uid}) {
		print qq|<b>$user->{username}</b><input type="hidden" name="name" value="$user->{username}">|;
	} else {
		print qq|<input type="text" name="name" value="$in{name}">|;
	}
	print qq|</td>
</tr>
<tr>
	<td>Vorname</td>
	<td><input type="text" name="firstname" value="$in{firstname}"></td>
</tr>
<tr>
	<td>Nachname</td>
	<td><input type="text" name="lastname" value="$in{lastname}"></td>
</tr>
<tr>
	<td>Email</td>
	<td><input type="text" name="email" value="$in{email}"></td>
</tr>
<tr>
	<td>Reminder</td>
	<td><input type="radio" name="reminder" value="1" $sel1>Ja &nbsp; <input type="radio" name="reminder" value="0" $sel0>Nein</td>
</tr>
<tr>
	<td>Level</td>
	<td><select name="level">|;

	my $stmt = "select id, name from Levels where status = 'F' order by seqnr";
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);
	foreach (@data) {
		my $sel = ($_->[0] == $in{level}) ? "selected" : "";
		print qq|<option value="$_->[0]" $sel> $_->[1]\n|;

	}
	print qq|</select></td>
</tr>
|;
	if ($user->{uid}) {
		print qq|
<tr>
	<td>Passwort</td>
	<td><input type="password" name="passwd1" value=""></td>
</tr>
<tr>
	<td>Pwd. best&auml;tigen</td>
	<td><input type="password" name="passwd2" value=""></td>
</tr>

|;
	}

	print qq|
<tr>
	<td></td>
	<td>
		<input type="submit" name="action_register" value="$submit">
	</td>
</tr>
</table>

</form>
</td></tr></table>
|;

}
