#!/usr/bin/perl -w

use strict;
use bytes;

use XML::XPath;

use lib "..";
use VocTrainer::Database;

if (!$ARGV[0] || !-e $ARGV[0]) {
	&usage;
	exit(0);
} 

my $xmlfile = $ARGV[0];

my $dbh = DB_open();

my $xp = XML::XPath->new(filename => $xmlfile);


## delete all articles;
print "\nDeleting Articles from DB.... \n";
DB_insertdata(STMT => "delete from Articles", DBH => $dbh);

## get all articles;
print "\nParsing Articles (/kvtml/article/e/*) .... \n";
my $articlelangs = $xp->find('/kvtml/article/e');
foreach my $node ($articlelangs->get_nodelist) {
	my $lang = $node->findvalue('@l');
	my $articles = $node->find('./*');
	foreach my $art ($articles->get_nodelist) {
		my $code = $art->getName();
		my $value = $art->string_value;
		my $stmt = "insert into Articles values (" . 
			$dbh->quote($code) . ", " . 
			$dbh->quote($lang) . ", " . 
			$dbh->quote($value) . ")";
		print "$stmt\n";
		DB_insertdata(STMT => $stmt, DBH => $dbh);
	}
}

## delete all articles;
print "\nDeleting Pronouns from DB.... \n";
DB_insertdata(STMT => "delete from Pronouns", DBH => $dbh);

## get all pronouns
print "\nParsing Pronouns (/kvtml/conjugation/e/*) .... \n";
my $pronounlangs = $xp->find('/kvtml/conjugation/e');
foreach my $node ($pronounlangs->get_nodelist) {
	my $lang = $node->findvalue('@l');
	my $pronouns = $node->find('./*');
	foreach my $p ($pronouns->get_nodelist) {
		my $common = $p->findvalue('@common');
		my $code = $p->getName();
		my $value = $p->string_value;
		my $stmt = "insert into Pronouns values (" . 
			$dbh->quote($code) . ", " . 
			$dbh->quote($lang) . ", " . 
			$dbh->quote($value) . ", " . 
			$dbh->quote($common) . ")";
		print "$stmt\n";
		DB_insertdata(STMT => $stmt, DBH => $dbh);
	}
}



my @types;

my $olang = $xp->findvalue('/kvtml/e[position() = 1]/o/@l');
print "O-Lang : $olang\n";
my $tlang = $xp->findvalue('/kvtml/e[position() = 1]/t/@l');
print "T-Lang : $tlang\n";

my $stmt = "delete from Entries";
DB_insertdata(STMT => $stmt, DBH => $dbh);

$stmt = "delete from Conjugations";
DB_insertdata(STMT => $stmt, DBH => $dbh);

my $id = 1;


## get all entries
my $entries = $xp->find('/kvtml/e');
foreach my $node ($entries->get_nodelist) {
	my $type = $node->findvalue('@t');
	if ($type) {
		push @types, $type;
	} else {
		$type = '';
	}

	my $os = $node->find('./o');
	my $ts = $node->find('./t');
	die "SHIT: zu viele os ..." if $#{$os} != 0;
	die "SHIT: zu viele ts ..." if $#{$ts} != 0;

#	print "O: " . XML::XPath::XMLParser::as_string($os->[0]) . "\n";
	my $o = &parse_entry($os->[0]);
#	print "T: " . XML::XPath::XMLParser::as_string($ts->[0]) . "\n";
	my $t = &parse_entry($ts->[0]);


	if ($o && $t) {
		$o->{typeid} = $t->{typeid} = $type;
		&insert_entry($o, $olang, $id);
		&insert_entry($t, $tlang, $id);
		$id++;
	}
	print "\n";
}



DB_close(DBH => $dbh);


sub insert_entry {
	my $e = shift;
	my $lang = shift;
	my $id = shift;

	my @columns = ('word', 'typeid', 'remark', 'pronounciation', 'antonyms', 'synonyms', 'example', 'paraphrase', 'entryusage');

	my $stmt = "insert into Entries values ($id, '$lang', ";
	$stmt .= join (", ", map { $dbh->quote($e->{$_}) } @columns);
	$stmt .= ")";
	print "STMT : $stmt\n";
	DB_insertdata(STMT => $stmt, DBH => $dbh);

	if ($e->{conjugations}) {
		my %conj = %{$e->{conjugations}};
		foreach my $code(keys %conj) {
			my $cstmt = "insert into Conjugations values ($id, '$lang', " . 
				$dbh->quote($code) . ", " . 
				$dbh->quote($conj{$code}) . ")";
			print "CONJ: $cstmt\n";
			DB_insertdata(STMT => $cstmt, DBH => $dbh);
		}
	}
}

sub parse_entry {
	my $e = shift;

	my $word = $e->string_value;
	return unless $word;

	my $remark = $e->findvalue('@r') || "";
	my $pronounciation = $e->findvalue('@p') || "";
	my $antonyms = $e->findvalue('@a') || "";
	my $synonyms = $e->findvalue('@y') || "";
	my $example = $e->findvalue('@x') || "";
	my $entryusage = $e->findvalue('@u') || "";
	my $paraphrase = $e->findvalue('@h') || "";

	my $conjugs = $e->find('./conjugation/*/*');
	my $conj;
	if ($conjugs) {
		my $tmp;
		foreach my $c ($conjugs->get_nodelist) {
#			print "Con: " . $c->getName() . " " . $c->string_value . "\n";
			my $code = $c->getName();
			my $value = $c->string_value;
			$conj->{$code} = $value;
			$tmp .= $value;
		}
		$word =~ s/$tmp//g;
		$word =~ s/[\n\r]//g;
		$word =~ s/^\s+//g;
		$word =~ s/\s+$//g;
	}
	print "WORD: $word\n";
	my %entry = (
		word => $word,
		remark => $remark,
		pronounciation => $pronounciation,
		antonyms => $antonyms,
		synonyms => $synonyms,
		example => $example,
		entryusage => $entryusage,
		paraphrase => $paraphrase,
		conjugations => $conj
	);
	return \%entry;
}

sub usage {
	print "\nUsage: ./import_kvtml.pl langfile.kvtml\n\n";

}
