#!/usr/bin/perl -w

use strict;

use CGI;

use VocTrainer::Database;
use VocTrainer::Layout;
use VocTrainer::Users;
use VocTrainer::Params;
use VocTrainer::Utils;


use constant NOLEVEL => 21;   # id of "No Level" - Level
# TODO get from db
my @languages = ('de', 'es');
my %langstrings = (
    "de" => "Deutsch",
    "es" => "Espaol"
);

my $q = new CGI;
my %in = GetParams(QUERY => $q);

my $dbh = DB_open();

my $user = VocTrainer::Users::CheckUser(DBH => $dbh);

print $q->header();
&VocTrainer::Layout::printhtmlheader(TITLE => "Vokabelliste" , USER => $user);
if (!$user->{uid}) {
	print qq|Bitte zuerst <a href="login.pl?back=voclist.pl"> einloggen</a>|;
} else {
	&listvocs;
}

DB_close(DBH => $dbh);

sub listvocs {

	my $uid = $user->{uid};
#	$uid = 2;
	my $stmt = "select e.entryid, count(e.entryid), sum(e.correct), sum(e.addpoint) from User_Lessions_Entries e, User_Lessions l where e.userlessionid = l.id and l.userid = $uid group by entryid";
	my @data = DB_getdata(STMT => $stmt, DBH => $dbh);

	my %ids = map { $_->[0] => { count => $_->[1], rate => (($_->[2]+$_->[3])/$_->[1]*100) } } @data;

	
	print qq|<table border="0" cellpadding="3" cellspacing="0" width="600">
<tr>
	<td colspan="2"><form>
		Vokabel anzeigen, die zu weniger als
		<select name="threshold" onChange="document.forms[0].submit();">
|;
	$in{threshold} ||= 50;
	for (my $t=10; $t <= 100; $t+=10) {
		my $sel = ($t == $in{threshold}) ? "selected" : "";
		print qq|<option value="$t" $sel> $t %</option>|;
	}
	print qq|
		</select> 
		richtig beantwortet wurden.<br>
		<br>
		Oder alle Vokabel auflisten, die immer <a href="voclist.pl?correct=1">richtig</a> beantwortet wurden.<br>
		<br>
		</form>
	</td>
</tr>
</table>

|;
	my @idstoprint;
	foreach (keys %ids) {
		if ($in{correct}) {
			push @idstoprint, $_ if ($ids{$_}->{rate} == 100);
		} else {
			push @idstoprint, $_ if ($ids{$_}->{rate} < $in{threshold});
		}
	}
	my $eids = join (",", @idstoprint);
    print qq|<table border="0" cellspacing="0" cellpadding="3" width="600">\n|;
	if ($eids) {

        my $stmt = "select id, langcode, word, article, synonyms from Entries where id in ($eids)";
        my @data = DB_getdata(STMT => $stmt, DBH => $dbh);

		my %entries;
		foreach (@data) {
			$entries{$_->[0]}->{$_->[1]} = {
				langcode => $_->[1],
				word => $_->[2],
				article => $_->[3],
				synonyms => $_->[4],
			}; 
		} 
		%entries = %{ VocTrainer::Utils::unicode2html(\%entries) };
		print qq|<tr class="tableheader">\n|;
		print qq|<td class="tableheader">#</td>|;
		foreach (@languages) {
			print qq|<td class="tableheader">$langstrings{$_}</td>\n|;
		}
		print qq|<td class="tableheader" align="center">Versuche</td>|;
		print qq|<td class="tableheader" align="right">Richtig</td>|;
		print qq|</tr>\n|;

		my $i = 0;
		foreach my $id (sort {$ids{$a}->{rate} <=> $ids{$b}->{rate} } @idstoprint) {
			my $row = "row" . ($i % 2);
			print qq|<tr class="$row">
	<td width="40">| . ($i + 1) . qq|</td>\n|;
			foreach my $lang (@languages) {
				print qq|\t<td width="200" valign="top">|;
				if ($entries{$id}->{$lang}->{article}) {
					print $entries{$id}->{$lang}->{article} . " ";
				}
				print $entries{$id}->{$lang}->{word};
				print " (Synonyme: " . $entries{$id}->{$lang}->{synonyms} . ")" if $entries{$id}->{$lang}->{synonyms};
				print qq|</td>\n|;
			}
			print qq|\t<td width="80" align="center">$ids{$id}->{count}</td>\n|;
			print qq|\t<td width="80" align="right">| . sprintf("%.2f", $ids{$id}->{rate}) . qq|&nbsp;%</td>\n|;
			print qq|</tr>\n|;
			$i++;
		}
	} else {
		print qq|
<tr>
	<td colspan="3" align="center">Keine Eintraege!!</td>
</tr>
|;

	}
}
